:li
/********************************************************************
 *
 * Motif-Programm zur Rechteckausgabe
 *
 * Quelle: BREDEX GmbH, Braunschweig
 *
 ********************************************************************/

/* Allgemeine Header-Dateien */
#include <stdio.h>
#include <stdlib.h>
#include <Xm/Xm.h>

/* Header-Dateien fuer verwendete Widgetklassen */
#include <Xm/DrawingA.h>

/* Klassenname der Applikation */
#define APPL_KLASSE    "XmBsp"

/* Vorwaerts-Deklarationen */
void initDrawingArea();
void redisplayDrawingAreaCB();
void exitCB();


main (argc, argv)
int   argc;
char* argv[];
{
    XtAppContext kontext;        /* Applikations-Umgebung */
    Widget       applShell;      /* Applikations-Shell */
    int		 linienbreite;   /* Linienbreite */

    /* Initialisierungen und Applikations-Shell erzeugen */
    applShell = XtAppInitialize (&kontext, APPL_KLASSE,
                                 (XrmOptionDescRec*)NULL, 0,
                                 &argc, argv, (String*)NULL,
                                 (Arg*)NULL, 0);

    /* (nicht verarbeiteter) Parameter ist Linienbreite */
    if (argc > 1) {
	linienbreite = atoi (argv[1]);
    }
    else {
	linienbreite = 0;	/* optimierte 1 */
    }

    /* Widgetbaum unter Applikations-Shell aufbauen */
    initDrawingArea (applShell, linienbreite);

    /* Widgetbaum realisieren */
    XtRealizeWidget (applShell);

    /* Hauptschleife mit Ereignisbehandlung */
    XtAppMainLoop (kontext);
}


void initDrawingArea (elternWidget, linienbreite)
Widget elternWidget;
int linienbreite;
{
    Widget da;		/* XmDrawingArea-Widget */
    GC gc;		/* Graphics-Context */
    XGCValues gcv;	/* Werte-Struktur dafuer */
    Pixel fg;		/* Vordergrundfarbe */
    Arg args[10];
    Cardinal n;

    /* XmDrawingArea-Widget erzeugen */
    n = 0;
    XtSetArg (args[n], XmNwidth, 50);  n++;
    XtSetArg (args[n], XmNheight, 25);  n++;
    da = XtCreateManagedWidget ("box", xmDrawingAreaWidgetClass,
				elternWidget, args, n);

    /* Vordergrundfarbe abfragen */
    n = 0;
    XtSetArg (args[n], XmNforeground, &fg);  n++;
    XtGetValues (da, args, n);

    /* Graphics-Context dazu erzeugen */
    gcv.foreground = fg;
    gcv.line_width = linienbreite;
    gc = XtGetGC (da, GCForeground | GCLineWidth, &gcv);

    /* Callback zum Ausgeben des Fensterinhalts installieren
     * (Rechtecke ausgeben)
     */
    XtAddCallback (da, XmNexposeCallback,
		   redisplayDrawingAreaCB, (XtPointer)gc);

    /* Exit-Callback zur Reaktion auf Eingaben installieren
     * (jede Maus- und Tastatur-Eingabe beendet das Programm)
     */
    XtAddCallback (da, XmNinputCallback,
		   exitCB, (XtPointer)NULL);
}

/* ARGSUSED */
void redisplayDrawingAreaCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (GC) */
XtPointer aufrufDaten; /* Aufruf-Daten (Callback-Informationen) */
{
    XmDrawingAreaCallbackStruct* evp;

    /* nur der letzte einer Folge von Expose-Events ist von
     * Interesse, da der Inhalt immer komplett ausgegeben wird
     */
    evp = (XmDrawingAreaCallbackStruct*)aufrufDaten;
    if (evp->event->xexpose.count != 0) {
	return;
    }

    /* Rechteck ausgeben */
    XDrawRectangle (XtDisplay(widget), XtWindow(widget), (GC)clientDaten,
		    10, 10, 10, 5);
}

/* ARGSUSED */
void exitCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (nicht verwendet) */
XtPointer aufrufDaten; /* Aufruf-Daten (nicht verwendet) */
{
    /* Programm mit fehlerfreiem Status beenden */ 
    exit (0);
}
